package at.ac.uibk.gitsearch.service.management;

import java.util.List;
import java.util.Map;

import javax.annotation.PostConstruct;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.actuate.health.Health;
import org.springframework.boot.actuate.health.HealthContributor;
import org.springframework.boot.actuate.health.HealthContributorRegistry;
import org.springframework.boot.actuate.health.HealthIndicator;
import org.springframework.stereotype.Service;

import at.ac.uibk.gitsearch.config.ApplicationProperties;
import at.ac.uibk.gitsearch.service.PluginManagementService;
import at.ac.uibk.gitsearch.service.PluginManagementService.PluginConfigWrapper;

@Service
//@Endpoint(id = "pluginHealth")
public class PluginHealthCheckRegistry {

	private static final Logger log = LoggerFactory.getLogger(PluginHealthCheckRegistry.class);

	@Autowired
	protected PluginManagementService pluginManagementService;
	@Autowired
	private ApplicationProperties applicationProperties;
	@Autowired
	private HealthContributorRegistry healthContributorRegistry;

	@PostConstruct
	protected void registerPluginHealth() {
		final List<String> registeredPlugins = applicationProperties.getRegisteredPlugins();
		if(registeredPlugins==null) {
			log.warn("No plugins defined?");
			return;
		}
		for (String configURL : registeredPlugins) {
			healthContributorRegistry.registerContributor(configURL,
					new PluginHealthCheck(configURL, pluginManagementService));
		}

	}

	public static class PluginHealthCheck implements HealthContributor, HealthIndicator {

		protected PluginManagementService pluginManagementService;
		private String configURL;

		public PluginHealthCheck(String configURL, PluginManagementService pluginManagementService) {
			super();
			this.configURL = configURL;
			this.pluginManagementService = pluginManagementService;
		}

		public Health health() {

			for (PluginConfigWrapper pluginConfig : pluginManagementService.getRegisteredPluginConfigs()) {
				if (configURL.equals(pluginConfig.getConfigURL())) {
					final String message = String.format("Running as %s with %d actions", pluginConfig.getPluginName(),
							pluginConfig.getActions().size());
					return Health.up().withDetail(pluginConfig.getPluginName(), message).build();
				}
			}
			for (Map.Entry<String, Exception> failures : pluginManagementService.getConfigFailures().entrySet()) {
				if (configURL.equals(failures.getKey())) {
					return Health.down(failures.getValue()).withDetail(configURL, "has failed").build();
				}
			}

			return Health.unknown().build();
		}
	}

}
