import { Injectable, OnDestroy, OnInit } from '@angular/core';
import { HttpResponse } from '@angular/common/http';
import { UserWatchListService } from 'app/entities/user-watch-list/user-watch-list.service';
import { IUserWatchList } from 'app/shared/model/user-watch-list.model';
import { AccountService } from 'app/core/auth/account.service';
import { Subscription } from 'rxjs';
import { Account } from 'app/core/user/account.model';


/**
 provides infrastructure for watchlists management.
 The manager is is charge to guarantee that the local watchlists and the watchlists on the server are in synchrony.
 If the user is not logged in, the watchlists are undefined.
 */
@Injectable({ providedIn: 'root' })
export class WatchlistManager implements OnInit, OnDestroy {

  account: Account | null = null;
  authSubscription?: Subscription;

    myWatchLists?: IUserWatchList[];
    currentWatchlist? = '';
    constructor(
        protected watchListService: UserWatchListService,
        private accountService: AccountService,

    ) {
        this.authSubscription = this.accountService.getAuthenticationState().subscribe(account => {
            this.account = account;
            if(this.accountService.isAuthenticated()) {
            this.watchListService.findMyWatchlists().subscribe((data: HttpResponse<IUserWatchList[]>) => {
                if (data.body) {
                    this.myWatchLists = data.body;
                    if(this.myWatchLists.length > 0) {
                        this.currentWatchlist = this.myWatchLists[0].name
                    }
                }

            })
                
            }
            else { this.myWatchLists = undefined; }
            });
    }
    
    public getMyWatchLists(): IUserWatchList[]| undefined {
        return this.myWatchLists;
    }

    ngOnInit(): void {
    }

  ngOnDestroy(): void {
    if (this.authSubscription) {
      this.authSubscription.unsubscribe();
    }
  }
}
